@@ -92,6 +92,7 @@ from ognibuild.upstream import (
     go_base_name,
     load_crate_info,
 )
+from ognibuild.debian.upstream_deps import get_project_wide_deps
 from ognibuild.vcs import dupe_vcs_tree
 
 from upstream_ontologist.guess import (
@@ -152,7 +153,6 @@ from .publish import (
     NoVcsLocation,
 )
 from .standards_version import latest_standards_version
-from .upstream_deps import get_project_wide_deps
 
 
 Kickstarter = Callable[[WorkingTree, str], None]
@@ -236,7 +236,7 @@ def versions_dict():
         'lintian-brush': lintian_brush.version_string,
         'debmutate': debmutate.version_string,
         'debian': debian.__version__,
-        'ognibuild': ognibuild.version_version,  # type: ignore
+        'ognibuild': ognibuild.version_string,
         'buildlog_consultant': buildlog_consultant.version_string,
         'upstream_ontologist': upstream_ontologist.version_string,
     }
